from datagen.modalities.descriptors import (
    VisualModalityDescriptor,
    VisualModality,
    TextualModalityDescriptor,
    TextualModality,
    ModalityPreProcess,
)

visual_modality = VisualModalityDescriptor
textual_modality = TextualModalityDescriptor
